<?php
/* --------------------------------------------------------------
   ShopOfflineApplicationTopExtender.inc.php 2019-08-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class ShopOfflineApplicationTopExtender extends ShopOfflineApplicationTopExtender_parent
{
    /**
     * Proceed.
     */
    public function proceed()
    {
        parent::proceed();
        
        if ($this->toShowOfflineMessage()) {
            header('Content-Type: text/html; charset=' . $_SESSION['language_charset']);
            echo $this->getOfflinePageHtml();
            exit;
        }
        
        define('SHOP_OFFLINE', !$this->releaseInfoMatchInstalledVersion());
    }
    
    
    /**
     * @return bool
     */
    protected function toShowOfflineMessage()
    {
        return !$this->userLoggedIn()
               && $this->shopIsOffline()
               && !$this->attemptLogin()
               && !$this->cssRequest()
               && !$this->gambioHubRequest()
               && !$this->appStoreRequest();
    }
    
    
    /**
     * @return string
     */
    protected function getGxShopVersion()
    {
        $gx_version = null;
        
        include DIR_FS_CATALOG . 'release_info.php';
        
        return $gx_version;
    }
    
    
    /**
     * @return string|null
     */
    protected function getInstalledShopVersion()
    {
        // do not use gm_get_conf() to avoid caching problems
        $query  = 'SELECT `gm_value` FROM `gm_configuration` WHERE `gm_key` = "INSTALLED_VERSION" LIMIT 1';
        $result = xtc_db_query($query);
        
        if (1 === xtc_db_num_rows($result)) {
            $row              = xtc_db_fetch_array($result);
            $installedVersion = $row['gm_value'];
        }
        
        return isset($installedVersion) ? $installedVersion : null;
    }
    
    
    /**
     * @return bool
     */
    protected function shopIsOffline()
    {
        return 'checked' === gm_get_conf('GM_SHOP_OFFLINE');
    }
    
    
    /**
     * @return bool
     */
    protected function userLoggedIn()
    {
        return '0' === $_SESSION['customers_status']['customers_status_id'];
    }
    
    
    /**
     * @return bool
     */
    protected function releaseInfoMatchInstalledVersion()
    {
        return $this->getGxShopVersion() === $this->getInstalledShopVersion();
    }
    
    
    /**
     * Checks if login is attempted.
     *
     * @return bool True if login is attempted.
     */
    protected function attemptLogin()
    {
        return isset($_POST['email_address'], $_POST['password'])
               || isset($_POST['2fa_token'], $_POST['cid'], $_POST['cemail'], $_POST['cpassword']);
    }
    
    
    /**
     * Returns true if the current request comes from GambioHub e.g (shop.php?do=shopinfo).
     *
     * @return bool
     */
    protected function gambioHubRequest()
    {
        if ('shop.php' !== basename(gm_get_env_info('SCRIPT_NAME'))) {
            return false;
        }
        
        $hubRequest = (string)array_search('shopinfo', array_map('strtolower', $_GET), true);
        
        return !empty($hubRequest);
    }
    
    
    /**
     * Returns true if the current request comes from stylesheet request.
     *
     * @return bool
     */
    protected function cssRequest()
    {
        if ('dynamic_theme_style.css.php' !== basename(gm_get_env_info('SCRIPT_NAME'))) {
            return false;
        }
        
        return true;
    }
    
    
    /**
     * @return string
     */
    protected function getOfflinePageHtml()
    {
        $favicon     = '';
        $logoManager = MainFactory::create('GMLogoManager', 'gm_logo_favicon');
        if ($logoManager->logo_use === '1') {
            $favicon = $logoManager->logo_path . $logoManager->logo_file;
        }
        
        $cssFileExtension = file_exists(DIR_FS_CATALOG . '.dev-environment') ? '' : '.min';
        
        if (StaticGXCoreLoader::getThemeControl()->isThemeSystemActive()) {
            if (file_exists(DIR_FS_CATALOG . 'public/theme/styles/system/main' . $cssFileExtension . '.css')) {
                $cssFile = 'public/theme/styles/system/main' . $cssFileExtension . '.css?bust='
                           . filemtime(DIR_FS_CATALOG . 'public/theme/styles/system/main' . $cssFileExtension . '.css');
            } else {
                $cssFile = 'dynamic_theme_style.css.php?theme=' . rawurlencode(StaticGXCoreLoader::getThemeControl()
                                                                                                 ->getCurrentTheme())
                           . '&bust=' . time();
            }
        } elseif (file_exists(DIR_FS_CATALOG . 'templates/' . CURRENT_TEMPLATE . '/main' . $cssFileExtension
                              . '.css')) {
            $cssFile = 'templates/' . rawurlencode(CURRENT_TEMPLATE) . '/main' . $cssFileExtension . '.css?bust='
                       . filemtime(DIR_FS_CATALOG . 'templates/' . CURRENT_TEMPLATE . '/main' . $cssFileExtension
                                   . '.css');
        } else {
            $cssFile = 'templates/' . rawurlencode(CURRENT_TEMPLATE) . '/gm_dynamic.css.php?current_template='
                       . rawurlencode(CURRENT_TEMPLATE) . '&bust=' . time();
        }
        
        $html = '<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" dir="ltr" lang="' . $_SESSION['language_code'] . '">
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=' . $_SESSION['language_charset'] . '" />
        <title>' . htmlspecialchars(STORE_NAME) . '</title>
        <base href="' . GM_HTTP_SERVER . DIR_WS_CATALOG . '" />
        <link rel="shortcut icon" href="' . $favicon . '" type="image/x-icon" />
        <link type="text/css" rel="stylesheet" href="' . $cssFile . '" />
    </head>
    <body>
        ' . gm_get_conf('GM_SHOP_OFFLINE_MSG') . '
    </body>
</html>';
        
        return $html;
    }
    
    
    /**
     * @return bool
     */
    private function appStoreRequest()
    {
        if ('shop.php' !== basename(gm_get_env_info('SCRIPT_NAME'))) {
            return false;
        }
        
        return isset($_GET['do']) && $_GET['do'] === 'AppStoreCallback/verify';
    }
}
